/**
 * @license
 * Copyright 2022 Google LLC
 * SPDX-License-Identifier: Apache-2.0
 */
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, state, kind, f) {
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a getter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot read private member from an object whose class did not declare it");
    return kind === "m" ? f : kind === "a" ? f.call(receiver) : f ? f.value : state.get(receiver);
};
var _NoOpExecution_instances, _NoOpExecution_execute;
import { BaseExecution } from './base.js';
import { Fingerprint } from '../fingerprint.js';
/**
 * Execution for a {@link NoOpScriptConfig}.
 */
export class NoOpExecution extends BaseExecution {
    constructor() {
        super(...arguments);
        _NoOpExecution_instances.add(this);
    }
    static execute(script, executor, logger) {
        var _a;
        return __classPrivateFieldGet((_a = new NoOpExecution(script, executor, logger)), _NoOpExecution_instances, "m", _NoOpExecution_execute).call(_a);
    }
}
_NoOpExecution_instances = new WeakSet(), _NoOpExecution_execute = async function _NoOpExecution_execute() {
    const dependencyFingerprints = await this.executeDependencies();
    if (!dependencyFingerprints.ok) {
        return dependencyFingerprints;
    }
    const fingerprint = await Fingerprint.compute(this.script, dependencyFingerprints.value);
    this.logger.log({
        script: this.script,
        type: 'success',
        reason: 'no-command',
    });
    return { ok: true, value: fingerprint };
};
//# sourceMappingURL=no-op.js.map