/**
 * @license
 * Copyright 2022 Google LLC
 * SPDX-License-Identifier: Apache-2.0
 */
var __classPrivateFieldSet = (this && this.__classPrivateFieldSet) || function (receiver, state, value, kind, f) {
    if (kind === "m") throw new TypeError("Private method is not writable");
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a setter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot write private member to an object whose class did not declare it");
    return (kind === "a" ? f.call(receiver, value) : f ? f.value = value : state.set(receiver, value)), value;
};
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, state, kind, f) {
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a getter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot read private member from an object whose class did not declare it");
    return kind === "m" ? f : kind === "a" ? f.call(receiver) : f ? f.value : state.get(receiver);
};
var _OneShotExecution_instances, _OneShotExecution_state, _OneShotExecution_cache, _OneShotExecution_workerPool, _OneShotExecution_ensureState, _OneShotExecution_execute, _OneShotExecution_shouldNotStart_get, _OneShotExecution_startCancelledEvent_get, _OneShotExecution_acquireSystemLockIfNeeded, _OneShotExecution_fingerprintIsFresh, _OneShotExecution_handleFresh, _OneShotExecution_handleCacheHit, _OneShotExecution_handleNeedsRun, _OneShotExecution_shouldClean, _OneShotExecution_anyInputFilesDeletedSinceLastRun, _OneShotExecution_saveToCacheIfPossible, _OneShotExecution_globOutputFilesBeforeRunning, _OneShotExecution_cachedOutputFilesBeforeRunning, _OneShotExecution_globOutputFilesAfterRunning, _OneShotExecution_cachedOutputFilesAfterRunning, _OneShotExecution_globOutputFiles, _OneShotExecution_dataDir_get, _OneShotExecution_fingerprintFilePath_get, _OneShotExecution_readPreviousFingerprint, _OneShotExecution_cachedPreviousFingerprint, _OneShotExecution_writeFingerprintFile, _OneShotExecution_prepareDataDir, _OneShotExecution_cleanOutput, _OneShotExecution_computeOutputManifest, _OneShotExecution_outputManifestIsFresh, _OneShotExecution_readPreviousOutputManifest, _OneShotExecution_writeOutputManifest, _OneShotExecution_outputManifestFilePath_get;
import * as pathlib from 'path';
import * as fs from 'fs/promises';
import { getScriptDataDir } from '../util/script-data-dir.js';
import { unreachable } from '../util/unreachable.js';
import { glob, GlobOutsideCwdError } from '../util/glob.js';
import { deleteEntries } from '../util/delete.js';
import lockfile from 'proper-lockfile';
import { ScriptChildProcess } from '../script-child-process.js';
import { BaseExecution } from './base.js';
import { Fingerprint } from '../fingerprint.js';
import { computeManifestEntry } from '../util/manifest.js';
/**
 * Execution for a {@link OneShotScriptConfig}.
 */
export class OneShotExecution extends BaseExecution {
    constructor(script, executor, workerPool, cache, logger) {
        super(script, executor, logger);
        _OneShotExecution_instances.add(this);
        _OneShotExecution_state.set(this, 'before-running');
        _OneShotExecution_cache.set(this, void 0);
        _OneShotExecution_workerPool.set(this, void 0);
        _OneShotExecution_cachedOutputFilesBeforeRunning.set(this, void 0);
        _OneShotExecution_cachedOutputFilesAfterRunning.set(this, void 0);
        _OneShotExecution_cachedPreviousFingerprint.set(this, void 0);
        __classPrivateFieldSet(this, _OneShotExecution_workerPool, workerPool, "f");
        __classPrivateFieldSet(this, _OneShotExecution_cache, cache, "f");
    }
    static execute(script, executor, workerPool, cache, logger) {
        var _a;
        return __classPrivateFieldGet((_a = new OneShotExecution(script, executor, workerPool, cache, logger)), _OneShotExecution_instances, "m", _OneShotExecution_execute).call(_a);
    }
}
_OneShotExecution_state = new WeakMap(), _OneShotExecution_cache = new WeakMap(), _OneShotExecution_workerPool = new WeakMap(), _OneShotExecution_cachedOutputFilesBeforeRunning = new WeakMap(), _OneShotExecution_cachedOutputFilesAfterRunning = new WeakMap(), _OneShotExecution_cachedPreviousFingerprint = new WeakMap(), _OneShotExecution_instances = new WeakSet(), _OneShotExecution_ensureState = function _OneShotExecution_ensureState(state) {
    if (__classPrivateFieldGet(this, _OneShotExecution_state, "f") !== state) {
        throw new Error(`Expected state ${state} but was ${__classPrivateFieldGet(this, _OneShotExecution_state, "f")}`);
    }
}, _OneShotExecution_execute = async function _OneShotExecution_execute() {
    __classPrivateFieldGet(this, _OneShotExecution_instances, "m", _OneShotExecution_ensureState).call(this, 'before-running');
    const dependencyFingerprints = await this.executeDependencies();
    if (!dependencyFingerprints.ok) {
        dependencyFingerprints.error.push(__classPrivateFieldGet(this, _OneShotExecution_instances, "a", _OneShotExecution_startCancelledEvent_get));
        return dependencyFingerprints;
    }
    // Significant time could have elapsed since we last checked because our
    // dependencies had to finish.
    if (__classPrivateFieldGet(this, _OneShotExecution_instances, "a", _OneShotExecution_shouldNotStart_get)) {
        return { ok: false, error: [__classPrivateFieldGet(this, _OneShotExecution_instances, "a", _OneShotExecution_startCancelledEvent_get)] };
    }
    return __classPrivateFieldGet(this, _OneShotExecution_instances, "m", _OneShotExecution_acquireSystemLockIfNeeded).call(this, async () => {
        // Note we must wait for dependencies to finish before generating the
        // cache key, because a dependency could create or modify an input file to
        // this script, which would affect the key.
        const fingerprint = await Fingerprint.compute(this.script, dependencyFingerprints.value);
        if (await __classPrivateFieldGet(this, _OneShotExecution_instances, "m", _OneShotExecution_fingerprintIsFresh).call(this, fingerprint)) {
            const manifestFresh = await __classPrivateFieldGet(this, _OneShotExecution_instances, "m", _OneShotExecution_outputManifestIsFresh).call(this);
            if (!manifestFresh.ok) {
                return { ok: false, error: [manifestFresh.error] };
            }
            if (manifestFresh.value) {
                return __classPrivateFieldGet(this, _OneShotExecution_instances, "m", _OneShotExecution_handleFresh).call(this, fingerprint);
            }
        }
        // Computing the fingerprint can take some time, and the next operation is
        // destructive. Another good opportunity to check if we should still
        // start.
        if (__classPrivateFieldGet(this, _OneShotExecution_instances, "a", _OneShotExecution_shouldNotStart_get)) {
            return { ok: false, error: [__classPrivateFieldGet(this, _OneShotExecution_instances, "a", _OneShotExecution_startCancelledEvent_get)] };
        }
        const cacheHit = fingerprint.data.cacheable
            ? await __classPrivateFieldGet(this, _OneShotExecution_cache, "f")?.get(this.script, fingerprint)
            : undefined;
        if (__classPrivateFieldGet(this, _OneShotExecution_instances, "a", _OneShotExecution_shouldNotStart_get)) {
            return { ok: false, error: [__classPrivateFieldGet(this, _OneShotExecution_instances, "a", _OneShotExecution_startCancelledEvent_get)] };
        }
        if (cacheHit !== undefined) {
            return __classPrivateFieldGet(this, _OneShotExecution_instances, "m", _OneShotExecution_handleCacheHit).call(this, cacheHit, fingerprint);
        }
        return __classPrivateFieldGet(this, _OneShotExecution_instances, "m", _OneShotExecution_handleNeedsRun).call(this, fingerprint);
    });
}, _OneShotExecution_shouldNotStart_get = function _OneShotExecution_shouldNotStart_get() {
    return this.executor.shouldStopStartingNewScripts;
}, _OneShotExecution_startCancelledEvent_get = function _OneShotExecution_startCancelledEvent_get() {
    return {
        script: this.script,
        type: 'failure',
        reason: 'start-cancelled',
    };
}, _OneShotExecution_acquireSystemLockIfNeeded = 
/**
 * Acquire a system-wide lock on the execution of this script, if the script
 * has any output files that require it.
 */
async function _OneShotExecution_acquireSystemLockIfNeeded(workFn) {
    if (this.script.output?.values.length === 0) {
        return workFn();
    }
    // The proper-lockfile library is designed to give an exclusive lock for a
    // *file*. That's slightly misaligned with our use-case, because there's no
    // particular file we need a lock for -- our lock is for the execution of
    // this script.
    //
    // We can still use the library, we just need to pick some arbitrary file to
    // ask it to lock for us. It actually errors if the file doesn't exist. So
    // we end up with a mostly pointless file, and an adjacent "<file>.lock"
    // directory that manages the lock (to acquire a lock, it does a mkdir for
    // "<file>.lock", which will atomically succeed or fail depending on whether
    // it already existed).
    //
    // TODO(aomarks) We could make our own implementation that directly takes a
    // directory to mkdir and doesn't care about the file. There are some nice
    // details proper-lockfile handles.
    const lockFile = pathlib.join(__classPrivateFieldGet(this, _OneShotExecution_instances, "a", _OneShotExecution_dataDir_get), 'lock');
    await fs.mkdir(pathlib.dirname(lockFile), { recursive: true });
    await fs.writeFile(lockFile, '');
    let loggedLocked = false;
    while (true) {
        try {
            const release = await lockfile.lock(lockFile, {
                // If this many milliseconds has elapsed since the lock mtime was last
                // updated, proper-lockfile will delete it and attempt to acquire the
                // lock again. This handles the case where a process holding the lock
                // hard-crashed.
                stale: 10000,
                // How frequently the mtime for the lock will be updated while it is
                // being held. This should be some smallish factor of "stale" so that
                // we're unlikely to appear stale when we're actually still working on
                // the script.
                update: 2000,
            });
            try {
                return await workFn();
            }
            finally {
                await release();
            }
        }
        catch (error) {
            if (error.code === 'ELOCKED') {
                if (!loggedLocked) {
                    // Only log this once.
                    this.logger.log({
                        script: this.script,
                        type: 'info',
                        detail: 'locked',
                    });
                    loggedLocked = true;
                }
                // Wait a moment before attempting to acquire the lock again.
                await new Promise((resolve) => setTimeout(resolve, 200));
                if (__classPrivateFieldGet(this, _OneShotExecution_instances, "a", _OneShotExecution_shouldNotStart_get)) {
                    return { ok: false, error: [__classPrivateFieldGet(this, _OneShotExecution_instances, "a", _OneShotExecution_startCancelledEvent_get)] };
                }
            }
            else {
                throw error;
            }
        }
    }
}, _OneShotExecution_fingerprintIsFresh = 
/**
 * Check whether the given fingerprint matches the current one from the
 * `.wireit` directory.
 */
async function _OneShotExecution_fingerprintIsFresh(fingerprint) {
    if (!fingerprint.data.cacheable) {
        return false;
    }
    const prevFingerprint = await __classPrivateFieldGet(this, _OneShotExecution_instances, "m", _OneShotExecution_readPreviousFingerprint).call(this);
    return prevFingerprint !== undefined && fingerprint.equal(prevFingerprint);
}, _OneShotExecution_handleFresh = function _OneShotExecution_handleFresh(fingerprint) {
    this.logger.log({
        script: this.script,
        type: 'success',
        reason: 'fresh',
    });
    return { ok: true, value: fingerprint };
}, _OneShotExecution_handleCacheHit = 
/**
 * Handle the outcome where the script was stale and we got a cache hit.
 */
async function _OneShotExecution_handleCacheHit(cacheHit, fingerprint) {
    // Delete the fingerprint and other files. It's important we do this before
    // restoring from cache, because we don't want to think that the previous
    // fingerprint is still valid when it no longer is.
    await __classPrivateFieldGet(this, _OneShotExecution_instances, "m", _OneShotExecution_prepareDataDir).call(this);
    // If we are restoring from cache, we should always delete existing output.
    // The purpose of "clean:false" and "clean:if-file-deleted" is to allow
    // tools with incremental build (like tsc --build) to work.
    //
    // However, this only applies when the tool is able to observe each
    // incremental change to the input files. When we restore from cache, we are
    // directly replacing the output files, and not invoking the tool at all, so
    // there is no way for the tool to do any cleanup.
    await __classPrivateFieldGet(this, _OneShotExecution_instances, "m", _OneShotExecution_cleanOutput).call(this);
    await cacheHit.apply();
    __classPrivateFieldSet(this, _OneShotExecution_state, 'after-running', "f");
    const writeFingerprintPromise = __classPrivateFieldGet(this, _OneShotExecution_instances, "m", _OneShotExecution_writeFingerprintFile).call(this, fingerprint);
    const outputFilesAfterRunning = await __classPrivateFieldGet(this, _OneShotExecution_instances, "m", _OneShotExecution_globOutputFilesAfterRunning).call(this);
    if (!outputFilesAfterRunning.ok) {
        return { ok: false, error: [outputFilesAfterRunning.error] };
    }
    if (outputFilesAfterRunning.value !== undefined) {
        await __classPrivateFieldGet(this, _OneShotExecution_instances, "m", _OneShotExecution_writeOutputManifest).call(this, await __classPrivateFieldGet(this, _OneShotExecution_instances, "m", _OneShotExecution_computeOutputManifest).call(this, outputFilesAfterRunning.value));
    }
    await writeFingerprintPromise;
    this.logger.log({
        script: this.script,
        type: 'success',
        reason: 'cached',
    });
    return { ok: true, value: fingerprint };
}, _OneShotExecution_handleNeedsRun = 
/**
 * Handle the outcome where the script was stale and we need to run it.
 */
async function _OneShotExecution_handleNeedsRun(fingerprint) {
    // Check if we should clean before we delete the fingerprint file, because
    // we sometimes need to read the previous fingerprint file to determine
    // this.
    const shouldClean = await __classPrivateFieldGet(this, _OneShotExecution_instances, "m", _OneShotExecution_shouldClean).call(this, fingerprint);
    // Delete the fingerprint and other files. It's important we do this before
    // starting the command, because we don't want to think that the previous
    // fingerprint is still valid when it no longer is.
    await __classPrivateFieldGet(this, _OneShotExecution_instances, "m", _OneShotExecution_prepareDataDir).call(this);
    if (shouldClean) {
        const result = await __classPrivateFieldGet(this, _OneShotExecution_instances, "m", _OneShotExecution_cleanOutput).call(this);
        if (!result.ok) {
            return { ok: false, error: [result.error] };
        }
    }
    const childResult = await __classPrivateFieldGet(this, _OneShotExecution_workerPool, "f").run(async () => {
        // Significant time could have elapsed since we last checked because of
        // parallelism limits.
        if (__classPrivateFieldGet(this, _OneShotExecution_instances, "a", _OneShotExecution_shouldNotStart_get)) {
            return { ok: false, error: __classPrivateFieldGet(this, _OneShotExecution_instances, "a", _OneShotExecution_startCancelledEvent_get) };
        }
        __classPrivateFieldSet(this, _OneShotExecution_state, 'running', "f");
        this.logger.log({
            script: this.script,
            type: 'info',
            detail: 'running',
        });
        const child = new ScriptChildProcess(
        // Unfortunately TypeScript doesn't automatically narrow this type
        // based on the undefined-command check we did just above.
        this.script);
        void this.executor.shouldKillRunningScripts.then(() => {
            child.kill();
        });
        child.stdout.on('data', (data) => {
            this.logger.log({
                script: this.script,
                type: 'output',
                stream: 'stdout',
                data,
            });
        });
        child.stderr.on('data', (data) => {
            this.logger.log({
                script: this.script,
                type: 'output',
                stream: 'stderr',
                data,
            });
        });
        const result = await child.completed;
        if (result.ok) {
            this.logger.log({
                script: this.script,
                type: 'success',
                reason: 'exit-zero',
            });
        }
        else {
            // This failure will propagate to the Executor eventually anyway, but
            // asynchronously.
            //
            // The problem with that is that when parallelism is constrained, the
            // next script waiting on this WorkerPool might start running before
            // the failure information propagates, because returning from this
            // function immediately unblocks the next worker.
            //
            // By directly notifying the Executor about the failure while we are
            // still inside the WorkerPool callback, we prevent this race
            // condition.
            this.executor.notifyFailure();
        }
        return result;
    });
    __classPrivateFieldSet(this, _OneShotExecution_state, 'after-running', "f");
    if (!childResult.ok) {
        return { ok: false, error: [childResult.error] };
    }
    const writeFingerprintPromise = __classPrivateFieldGet(this, _OneShotExecution_instances, "m", _OneShotExecution_writeFingerprintFile).call(this, fingerprint);
    const outputFilesAfterRunning = await __classPrivateFieldGet(this, _OneShotExecution_instances, "m", _OneShotExecution_globOutputFilesAfterRunning).call(this);
    if (!outputFilesAfterRunning.ok) {
        return { ok: false, error: [outputFilesAfterRunning.error] };
    }
    if (outputFilesAfterRunning.value !== undefined) {
        await __classPrivateFieldGet(this, _OneShotExecution_instances, "m", _OneShotExecution_writeOutputManifest).call(this, await __classPrivateFieldGet(this, _OneShotExecution_instances, "m", _OneShotExecution_computeOutputManifest).call(this, outputFilesAfterRunning.value));
    }
    await writeFingerprintPromise;
    if (fingerprint.data.cacheable) {
        const result = await __classPrivateFieldGet(this, _OneShotExecution_instances, "m", _OneShotExecution_saveToCacheIfPossible).call(this, fingerprint);
        if (!result.ok) {
            return { ok: false, error: [result.error] };
        }
    }
    return { ok: true, value: fingerprint };
}, _OneShotExecution_shouldClean = async function _OneShotExecution_shouldClean(fingerprint) {
    const cleanValue = this.script.clean;
    switch (cleanValue) {
        case true: {
            return true;
        }
        case false: {
            return false;
        }
        case 'if-file-deleted': {
            const prevFingerprint = await __classPrivateFieldGet(this, _OneShotExecution_instances, "m", _OneShotExecution_readPreviousFingerprint).call(this);
            if (prevFingerprint === undefined) {
                // If we don't know the previous fingerprint, then we can't know
                // whether any input files were removed. It's safer to err on the
                // side of cleaning.
                return true;
            }
            return __classPrivateFieldGet(this, _OneShotExecution_instances, "m", _OneShotExecution_anyInputFilesDeletedSinceLastRun).call(this, fingerprint, prevFingerprint);
        }
        default: {
            throw new Error(`Unhandled clean setting: ${unreachable(cleanValue)}`);
        }
    }
}, _OneShotExecution_anyInputFilesDeletedSinceLastRun = function _OneShotExecution_anyInputFilesDeletedSinceLastRun(curFingerprint, prevFingerprint) {
    const curFiles = Object.keys(curFingerprint.data.files);
    const prevFiles = Object.keys(prevFingerprint.data.files);
    if (curFiles.length < prevFiles.length) {
        return true;
    }
    const newFilesSet = new Set(curFiles);
    for (const oldFile of prevFiles) {
        if (!newFilesSet.has(oldFile)) {
            return true;
        }
    }
    return false;
}, _OneShotExecution_saveToCacheIfPossible = 
/**
 * Save the current output files to the configured cache if possible.
 */
async function _OneShotExecution_saveToCacheIfPossible(fingerprint) {
    if (__classPrivateFieldGet(this, _OneShotExecution_cache, "f") === undefined) {
        return { ok: true, value: undefined };
    }
    const paths = await __classPrivateFieldGet(this, _OneShotExecution_instances, "m", _OneShotExecution_globOutputFilesAfterRunning).call(this);
    if (!paths.ok) {
        return paths;
    }
    if (paths.value === undefined) {
        return { ok: true, value: undefined };
    }
    await __classPrivateFieldGet(this, _OneShotExecution_cache, "f").set(this.script, fingerprint, paths.value);
    return { ok: true, value: undefined };
}, _OneShotExecution_globOutputFilesBeforeRunning = function _OneShotExecution_globOutputFilesBeforeRunning() {
    __classPrivateFieldGet(this, _OneShotExecution_instances, "m", _OneShotExecution_ensureState).call(this, 'before-running');
    return (__classPrivateFieldSet(this, _OneShotExecution_cachedOutputFilesBeforeRunning, __classPrivateFieldGet(this, _OneShotExecution_cachedOutputFilesBeforeRunning, "f") ?? __classPrivateFieldGet(this, _OneShotExecution_instances, "m", _OneShotExecution_globOutputFiles).call(this), "f"));
}, _OneShotExecution_globOutputFilesAfterRunning = function _OneShotExecution_globOutputFilesAfterRunning() {
    __classPrivateFieldGet(this, _OneShotExecution_instances, "m", _OneShotExecution_ensureState).call(this, 'after-running');
    return (__classPrivateFieldSet(this, _OneShotExecution_cachedOutputFilesAfterRunning, __classPrivateFieldGet(this, _OneShotExecution_cachedOutputFilesAfterRunning, "f") ?? __classPrivateFieldGet(this, _OneShotExecution_instances, "m", _OneShotExecution_globOutputFiles).call(this), "f"));
}, _OneShotExecution_globOutputFiles = 
/**
 * Glob the output files for this script, or return undefined if output files
 * are not defined.
 */
async function _OneShotExecution_globOutputFiles() {
    if (this.script.output === undefined) {
        return { ok: true, value: undefined };
    }
    try {
        return {
            ok: true,
            value: await glob(this.script.output.values, {
                cwd: this.script.packageDir,
                followSymlinks: false,
                includeDirectories: true,
                expandDirectories: true,
                throwIfOutsideCwd: true,
            }),
        };
    }
    catch (error) {
        if (error instanceof GlobOutsideCwdError) {
            // TODO(aomarks) It would be better to do this in the Analyzer by
            // looking at the output glob patterns. See
            // https://github.com/google/wireit/issues/64.
            return {
                ok: false,
                error: {
                    type: 'failure',
                    reason: 'invalid-config-syntax',
                    script: this.script,
                    diagnostic: {
                        severity: 'error',
                        message: `Output files must be within the package: ${error.message}`,
                        location: {
                            file: this.script.declaringFile,
                            range: {
                                offset: this.script.output.node.offset,
                                length: this.script.output.node.length,
                            },
                        },
                    },
                },
            };
        }
        throw error;
    }
}, _OneShotExecution_dataDir_get = function _OneShotExecution_dataDir_get() {
    return getScriptDataDir(this.script);
}, _OneShotExecution_fingerprintFilePath_get = function _OneShotExecution_fingerprintFilePath_get() {
    return pathlib.join(__classPrivateFieldGet(this, _OneShotExecution_instances, "a", _OneShotExecution_dataDir_get), 'fingerprint');
}, _OneShotExecution_readPreviousFingerprint = 
/**
 * Read this script's previous fingerprint from `fingerprint` file in the
 * `.wireit` directory. Cached after first call.
 */
async function _OneShotExecution_readPreviousFingerprint() {
    if (__classPrivateFieldGet(this, _OneShotExecution_cachedPreviousFingerprint, "f") === undefined) {
        __classPrivateFieldSet(this, _OneShotExecution_cachedPreviousFingerprint, (async () => {
            try {
                return Fingerprint.fromString((await fs.readFile(__classPrivateFieldGet(this, _OneShotExecution_instances, "a", _OneShotExecution_fingerprintFilePath_get), 'utf8')));
            }
            catch (error) {
                if (error.code === 'ENOENT') {
                    return undefined;
                }
                throw error;
            }
        })(), "f");
    }
    return __classPrivateFieldGet(this, _OneShotExecution_cachedPreviousFingerprint, "f");
}, _OneShotExecution_writeFingerprintFile = 
/**
 * Write this script's fingerprint file.
 */
async function _OneShotExecution_writeFingerprintFile(fingerprint) {
    await fs.mkdir(__classPrivateFieldGet(this, _OneShotExecution_instances, "a", _OneShotExecution_dataDir_get), { recursive: true });
    await fs.writeFile(__classPrivateFieldGet(this, _OneShotExecution_instances, "a", _OneShotExecution_fingerprintFilePath_get), fingerprint.string, 'utf8');
}, _OneShotExecution_prepareDataDir = 
/**
 * Delete the fingerprint and other files for this script from the previous
 * run, and ensure the data directory exists.
 */
async function _OneShotExecution_prepareDataDir() {
    await Promise.all([
        fs.rm(__classPrivateFieldGet(this, _OneShotExecution_instances, "a", _OneShotExecution_fingerprintFilePath_get), { force: true }),
        fs.mkdir(__classPrivateFieldGet(this, _OneShotExecution_instances, "a", _OneShotExecution_dataDir_get), { recursive: true }),
    ]);
}, _OneShotExecution_cleanOutput = 
/**
 * Delete all files matched by this script's "output" glob patterns.
 */
async function _OneShotExecution_cleanOutput() {
    const files = await __classPrivateFieldGet(this, _OneShotExecution_instances, "m", _OneShotExecution_globOutputFilesBeforeRunning).call(this);
    if (!files.ok) {
        return files;
    }
    if (files.value === undefined) {
        return { ok: true, value: undefined };
    }
    await deleteEntries(files.value);
    return { ok: true, value: undefined };
}, _OneShotExecution_computeOutputManifest = 
/**
 * Compute the output manifest for this script, which is the sorted list of
 * all output filenames, along with filesystem metadata that we assume is good
 * enough for checking that a file hasn't changed: ctime, mtime, and bytes.
 */
async function _OneShotExecution_computeOutputManifest(outputEntries) {
    outputEntries.sort((a, b) => a.path.localeCompare(b.path));
    const stats = await Promise.all(outputEntries.map((entry) => fs.lstat(entry.path)));
    const manifest = {};
    for (let i = 0; i < outputEntries.length; i++) {
        manifest[outputEntries[i].path] = computeManifestEntry(stats[i]);
    }
    return JSON.stringify(manifest);
}, _OneShotExecution_outputManifestIsFresh = 
/**
 * Check whether the current manifest of output files matches the one from the
 * `.wireit` directory.
 */
async function _OneShotExecution_outputManifestIsFresh() {
    const oldManifestPromise = __classPrivateFieldGet(this, _OneShotExecution_instances, "m", _OneShotExecution_readPreviousOutputManifest).call(this);
    const outputFilesBeforeRunning = await __classPrivateFieldGet(this, _OneShotExecution_instances, "m", _OneShotExecution_globOutputFilesBeforeRunning).call(this);
    if (!outputFilesBeforeRunning.ok) {
        return outputFilesBeforeRunning;
    }
    if (outputFilesBeforeRunning.value === undefined) {
        return { ok: true, value: false };
    }
    const newManifest = await __classPrivateFieldGet(this, _OneShotExecution_instances, "m", _OneShotExecution_computeOutputManifest).call(this, outputFilesBeforeRunning.value);
    const oldManifest = await oldManifestPromise;
    if (oldManifest === undefined) {
        return { ok: true, value: false };
    }
    const equal = newManifest === oldManifest;
    if (!equal) {
        this.logger.log({
            script: this.script,
            type: 'info',
            detail: 'output-modified',
        });
    }
    return { ok: true, value: equal };
}, _OneShotExecution_readPreviousOutputManifest = 
/**
 * Read this script's previous output manifest file from the `manifest` file
 * in the `.wireit` directory. Not cached.
 */
async function _OneShotExecution_readPreviousOutputManifest() {
    try {
        return (await fs.readFile(__classPrivateFieldGet(this, _OneShotExecution_instances, "a", _OneShotExecution_outputManifestFilePath_get), 'utf8'));
    }
    catch (error) {
        if (error.code === 'ENOENT') {
            return undefined;
        }
        throw error;
    }
}, _OneShotExecution_writeOutputManifest = 
/**
 * Write this script's output manifest file.
 */
async function _OneShotExecution_writeOutputManifest(outputManifest) {
    await fs.mkdir(__classPrivateFieldGet(this, _OneShotExecution_instances, "a", _OneShotExecution_dataDir_get), { recursive: true });
    await fs.writeFile(__classPrivateFieldGet(this, _OneShotExecution_instances, "a", _OneShotExecution_outputManifestFilePath_get), outputManifest, 'utf8');
}, _OneShotExecution_outputManifestFilePath_get = function _OneShotExecution_outputManifestFilePath_get() {
    return pathlib.join(__classPrivateFieldGet(this, _OneShotExecution_instances, "a", _OneShotExecution_dataDir_get), 'manifest');
};
//# sourceMappingURL=one-shot.js.map